/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/12/26
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for 
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

/*
 *@Note
 *USART Print debugging routine:
 *USART1_Tx(PA9).
 *This example demonstrates using USART1(PA9) as a print debug port output.
 *
 */

#include "debug.h"

#include <FreeRTOS.h>
#include <task.h>

#include "USBPD.h"
#include "ch32_pd.h"

/* Global typedef */

/* Global define */

/* Global Variable */
// StaticTask_t xTaskBuffer;
// StackType_t xStack[256];
TaskHandle_t xHandle = NULL;

/*********************************************************************
 * @fn      GPIO_Toggle_INIT
 *
 * @brief   Initializes GPIOA.0/1
 *
 * @return  none
 */
void GPIO_Toggle_INIT(void)
{
	GPIO_InitTypeDef GPIO_InitStructure = {0};

	RCC_PB2PeriphClockCmd(RCC_PB2Periph_GPIOA, ENABLE);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

/*********************************************************************
 * @fn      task2_task
 *
 * @brief   task2 program.
 *
 * @param  *pvParameters - Parameters point of task2
 *
 * @return  none
 */
void task2_task(void *pvParameters)
{
	while (1)
	{
		printf("task2 entry\r\n");
		GPIO_ResetBits(GPIOA, GPIO_Pin_8);
		vTaskDelay(500);
		GPIO_SetBits(GPIOA, GPIO_Pin_8);
		vTaskDelay(500);
	}
}

void vApplicationStackOverflowHook(TaskHandle_t xTask,
								   char *pcTaskName)
{
	(void)xTask;
	(void)pcTaskName;

	printf("stack overflow %s\r\n", pcTaskName);
	GPIO_SetBits(GPIOA, GPIO_Pin_8);
	while (1)
		;
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(921600);
    printf("SystemClk:%ld\r\n", SystemCoreClock);
	printf("ChipID:%08lx\r\n", DBGMCU_GetCHIPID());
	printf("FreeRTOS Kernel Version:%s\r\n", tskKERNEL_VERSION_NUMBER);
    printf("This is printf example\r\n");

    GPIO_Toggle_INIT();

    // xHandle = xTaskCreateStatic(POWTask, "USBPD", 256, NULL, 10, xStack, &xTaskBuffer);
	xTaskCreate(POWTask, "USBPD", 256, NULL, 10, &xHandle);

	// USBPowerDelivery::start();
	// ucpd_ch32_init();

    xTaskCreate(task2_task, "task2", 256, NULL, 5, NULL);

	// while (1)
	// {
	// 	printf("hello world\r\n");
	// 	Delay_Ms(1000);
	// }

    if (xHandle == NULL)
    {
        printf("Task create failed\r\n");
    }

    printf("Task create success\r\n");

    vTaskStartScheduler();

    while(1)
    {
        printf("Should not be here\r\n");
    }
}
